import { Inject, Injectable, Optional, } from '@angular/core';
import { BehaviorSubject, fromEvent, } from 'rxjs';
import { map, shareReplay, switchMap, takeUntil } from 'rxjs/operators';
import { onStrategy } from '../utils/onStrategy';
import { DEFAULT_STRATEGIES } from './default-strategies';
import { RX_CUSTOM_STRATEGIES } from './tokens/custom-strategies-token';
import { RX_PRIMARY_STRATEGY } from './tokens/default-primary-strategy-token';
import * as i0 from "@angular/core";
import * as i1 from "./tokens/default-primary-strategy-token";
import * as i2 from "./tokens/custom-strategies-token";
export class StrategyProvider {
    constructor(defaultStrategy, customStrategies) {
        this._strategies$ = new BehaviorSubject(undefined);
        this._primaryStrategy$ = new BehaviorSubject(undefined);
        this.primaryStrategy$ = this._primaryStrategy$.asObservable();
        this.strategies$ = this._strategies$.asObservable();
        this.strategyNames$ = this.strategies$.pipe(map((strategies) => Object.values(strategies).map((s) => s.name)), shareReplay({ bufferSize: 1, refCount: true }));
        const strats = Object.assign(Object.assign({}, DEFAULT_STRATEGIES), (customStrategies || []).reduce((acc, curr) => (Object.assign(Object.assign({}, acc), curr)), {}));
        this._strategies$.next(strats);
        this.primaryStrategy = defaultStrategy;
    }
    get strategies() {
        return this._strategies$.getValue();
    }
    get strategyNames() {
        return Object.values(this.strategies).map((s) => s.name);
    }
    get primaryStrategy() {
        return this._primaryStrategy$.getValue().name;
    }
    set primaryStrategy(strategyName) {
        this._primaryStrategy$.next(this.strategies[strategyName]);
    }
    scheduleWith(work, options) {
        const strategy = this.strategies[(options === null || options === void 0 ? void 0 : options.strategy) || this.primaryStrategy];
        const scope = (options === null || options === void 0 ? void 0 : options.scope) || {};
        const _work = getWork(work, options === null || options === void 0 ? void 0 : options.patchZone);
        return (o$) => o$.pipe(switchMap((v) => onStrategy(v, strategy, (_v) => {
            _work(_v);
        }, { scope })));
    }
    schedule(work, options) {
        const strategy = this.strategies[(options === null || options === void 0 ? void 0 : options.strategy) || this.primaryStrategy];
        const scope = (options === null || options === void 0 ? void 0 : options.scope) || {};
        const _work = getWork(work, options === null || options === void 0 ? void 0 : options.patchZone);
        let returnVal;
        return onStrategy(null, strategy, () => {
            returnVal = _work();
        }, { scope }).pipe(map(() => returnVal));
    }
    scheduleCD(cdRef, options) {
        const strategy = this.strategies[(options === null || options === void 0 ? void 0 : options.strategy) || this.primaryStrategy];
        const scope = (options === null || options === void 0 ? void 0 : options.scope) || cdRef;
        const abC = (options === null || options === void 0 ? void 0 : options.abortCtrl) || new AbortController();
        onStrategy(null, strategy, () => {
            strategy.work(cdRef, scope);
            if (options === null || options === void 0 ? void 0 : options.afterCD) {
                options.afterCD();
            }
        }, { scope })
            .pipe(takeUntil(fromEvent(abC.signal, 'abort')))
            .subscribe();
        return abC;
    }
}
StrategyProvider.ɵprov = i0.ɵɵdefineInjectable({ factory: function StrategyProvider_Factory() { return new StrategyProvider(i0.ɵɵinject(i1.RX_PRIMARY_STRATEGY), i0.ɵɵinject(i2.RX_CUSTOM_STRATEGIES, 8)); }, token: StrategyProvider, providedIn: "root" });
StrategyProvider.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
StrategyProvider.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [RX_PRIMARY_STRATEGY,] }] },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [RX_CUSTOM_STRATEGIES,] }] }
];
function getWork(work, patchZone) {
    let _work = work;
    if (patchZone) {
        _work = (args) => patchZone.run(() => work(args));
    }
    return _work;
}
//# sourceMappingURL=strategy-provider.service.js.map